﻿// Copyright (c) Autofac Project. All rights reserved.
// Licensed under the MIT License. See LICENSE in the project root for license information.

using System;
using Moq;
using Xunit;

namespace Autofac.Extras.AggregateService.Test
{
    public class AggregateServiceGenericsFixture
    {
        private readonly IContainer _container;

        public AggregateServiceGenericsFixture()
        {
            var builder = new ContainerBuilder();
            builder.RegisterAggregateService<IOpenGenericAggregate>();
            builder.RegisterGeneric(typeof(OpenGenericImpl<>))
                .As(typeof(IOpenGeneric<>));
            builder.RegisterGeneric(typeof(OpenGenericPassThroughImpl<>))
                .As(typeof(IPassThroughOpenGeneric<>));

            _container = builder.Build();
        }

        /// <summary>
        /// Attempts to resolve an open generic by a method call.
        /// </summary>
        [Fact]
        public void Method_ResolveOpenGeneric()
        {
            var aggregateService = _container.Resolve<IOpenGenericAggregate>();

            var generic = aggregateService.GetOpenGeneric<object>();
            Assert.NotNull(generic);

            var ungeneric = aggregateService.GetResolvedGeneric();
            Assert.NotNull(ungeneric);
            Assert.NotSame(generic, ungeneric);
        }

        [Fact]
        public void Method_UnpackOpenGeneric()
        {
            var aggregateService = _container.Resolve<IOpenGenericAggregate>();

            var generic = aggregateService.GetOpenGeneric<It.IsAnyType>();

            // works
            var passThroughFunc = _container.Resolve<Func<IOpenGeneric<It.IsAnyType>, IPassThroughOpenGeneric<It.IsAnyType>>>();
            var genericFromPassThroughFunc = passThroughFunc(generic);
            Assert.Same(generic, genericFromPassThroughFunc.OpenGeneric);

            // doesn't work
            var genericPassThrough = aggregateService.PassThroughOpenGeneric(generic);
            Assert.Same(generic, genericPassThrough.OpenGeneric);
        }
    }
}
